@@ -238,6 +238,8 @@ static void __Pyx_Generator_Replace_StopIteration(void) {
     PyObject *exc, *val, *tb;
     // Chain exceptions by moving StopIteration to exc_info before creating the RuntimeError.
     // In Py2.x, no chaining happens, but the exception still stays visible in exc_info.
+    __Pyx_PyThreadState_declare
+    __Pyx_PyThreadState_assign
     __Pyx_GetException(&exc, &val, &tb);
     Py_XDECREF(exc);
     Py_XDECREF(val);
@@ -306,7 +308,9 @@ static int __pyx_Generator_init(void); /*proto*/
 
 
 //////////////////// CoroutineBase ////////////////////
+//@substitute: naming
 //@requires: Exceptions.c::PyErrFetchRestore
+//@requires: Exceptions.c::PyThreadStateGet
 //@requires: Exceptions.c::SwapException
 //@requires: Exceptions.c::RaiseException
 //@requires: ObjectHandling.c::PyObjectCallMethod1
@@ -332,6 +336,8 @@ static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
 static int __Pyx_PyGen_FetchStopIterationValue(PyObject **pvalue) {
     PyObject *et, *ev, *tb;
     PyObject *value = NULL;
+    __Pyx_PyThreadState_declare
+    __Pyx_PyThreadState_assign
 
     __Pyx_ErrFetch(&et, &ev, &tb);
 
@@ -450,6 +456,7 @@ int __Pyx_Coroutine_CheckRunning(__pyx_CoroutineObject *gen) {
 static CYTHON_INLINE
 PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value) {
     PyObject *retval;
+    __Pyx_PyThreadState_declare
 
     assert(!self->is_running);
 
@@ -467,7 +474,7 @@ PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value) {
         return NULL;
     }
 
-
+    __Pyx_PyThreadState_assign
     if (value) {
 #if CYTHON_COMPILING_IN_PYPY
         // FIXME: what to do in PyPy?
@@ -475,13 +482,12 @@ PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value) {
         // Generators always return to their most recent caller, not
         // necessarily their creator.
         if (self->exc_traceback) {
-            PyThreadState *tstate = PyThreadState_GET();
             PyTracebackObject *tb = (PyTracebackObject *) self->exc_traceback;
             PyFrameObject *f = tb->tb_frame;
 
-            Py_XINCREF(tstate->frame);
+            Py_XINCREF($local_tstate_cname->frame);
             assert(f->f_back == NULL);
-            f->f_back = tstate->frame;
+            f->f_back = $local_tstate_cname->frame;
         }
 #endif
         __Pyx_ExceptionSwap(&self->exc_type, &self->exc_value,
@@ -798,6 +804,7 @@ static void __Pyx_Coroutine_del(PyObject *self) {
     PyObject *res;
     PyObject *error_type, *error_value, *error_traceback;
     __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
+    __Pyx_PyThreadState_declare
 
     if (gen->resume_label <= 0)
         return ;
@@ -809,6 +816,7 @@ static void __Pyx_Coroutine_del(PyObject *self) {
 #endif
 
     // Save the current exception, if any.
+    __Pyx_PyThreadState_assign
     __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
 
     res = __Pyx_Coroutine_Close(self);
@@ -951,11 +959,7 @@ typedef struct {
 } __pyx_CoroutineAwaitObject;
 
 static void __Pyx_CoroutineAwait_dealloc(PyObject *self) {
-#if CYTHON_COMPILING_IN_CPYTHON
-    _PyObject_GC_UNTRACK(self);
-#else
     PyObject_GC_UnTrack(self);
-#endif
     Py_CLEAR(((__pyx_CoroutineAwaitObject*)self)->coroutine);
     PyObject_GC_Del(self);
 }
@@ -1076,7 +1080,7 @@ static CYTHON_INLINE PyObject *__Pyx__Coroutine_await(PyObject *coroutine) {
     Py_INCREF(coroutine);
     await->coroutine = coroutine;
 #if CYTHON_COMPILING_IN_CPYTHON
-    _PyObject_GC_TRACK(await);
+    PyObject_GC_Track(await);
 #endif
     return (PyObject*)await;
 }
@@ -1098,7 +1102,7 @@ static void __Pyx_Coroutine_check_and_dealloc(PyObject *self) {
 #if PY_VERSION_HEX >= 0x03030000 || defined(PyErr_WarnFormat)
         PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname);
 #else
-        PyObject *msg;
+        {PyObject *msg;
         char *cmsg;
         #if CYTHON_COMPILING_IN_PYPY
         msg = NULL;
@@ -1139,7 +1143,7 @@ static void __Pyx_Coroutine_check_and_dealloc(PyObject *self) {
         #endif
         if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, cmsg, 1) < 0))
             PyErr_WriteUnraisable(self);
-        Py_XDECREF(msg);
+        Py_XDECREF(msg);}
 #endif
         PyObject_GC_Track(self);
     }
@@ -1386,6 +1390,7 @@ static void __Pyx__ReturnWithStopIteration(PyObject* value); /*proto*/
 
 /////////////// ReturnWithStopIteration ///////////////
 //@requires: Exceptions.c::PyErrFetchRestore
+//@requires: Exceptions.c::PyThreadStateGet
 //@substitute: naming
 
 // 1) Instantiating an exception just to pass back a value is costly.
@@ -1396,6 +1401,7 @@ static void __Pyx__ReturnWithStopIteration(PyObject* value); /*proto*/
 static void __Pyx__ReturnWithStopIteration(PyObject* value) {
     PyObject *exc, *args;
 #if CYTHON_COMPILING_IN_CPYTHON
+    __Pyx_PyThreadState_declare
     if ((PY_VERSION_HEX >= 0x03030000 && PY_VERSION_HEX < 0x030500B1) || PyTuple_Check(value)) {
         args = PyTuple_New(1);
         if (unlikely(!args)) return;
@@ -1409,7 +1415,8 @@ static void __Pyx__ReturnWithStopIteration(PyObject* value) {
         Py_INCREF(value);
         exc = value;
     }
-    if (!PyThreadState_GET()->exc_type) {
+    __Pyx_PyThreadState_assign
+    if (!$local_tstate_cname->exc_type) {
         // no chaining needed => avoid the overhead in PyErr_SetObject()
         Py_INCREF(PyExc_StopIteration);
         __Pyx_ErrRestore(PyExc_StopIteration, exc, NULL);
@@ -1700,11 +1707,7 @@ static PyTypeObject __Pyx__PyExc_StopAsyncIteration_type = {
     0,                                  /*tp_print*/
     0,                                  /*tp_getattr*/
     0,                                  /*tp_setattr*/
-#if PY_MAJOR_VERSION < 3
-    0,                                  /*tp_compare*/
-#else
-    0,                                  /*reserved*/
-#endif
+    0,                                  /*tp_compare / reserved*/
     0,                                  /*tp_repr*/
     0,                                  /*tp_as_number*/
     0,                                  /*tp_as_sequence*/
